from . import _version
from .models import (Author, Color, EmbeddedMessage, Field,  # noqa: F401, F403
                     Footer)
from .plugin import BotDiscordPlugin  # noqa: F401, F403

__version__ = _version.get_versions()['version']



import os
from typing import Dict, List, Union

from discord_webhook import DiscordEmbed, DiscordWebhook
from requests import Response

from .models import Color, EmbeddedMessage


class BotDiscordPlugin:
    def __init__(self, urls: Union[str, List[str]], username: str = None, **kwargs):
        """
        BotDiscordPlugin

        Args:
            urls (list or str): Webhook urls.
            username (str): The bot username
        """
        self._urls = urls
        self._username = username

    @property
    def webhook(self, **kwargs) -> DiscordWebhook:
        """
        Returns the discord-webhook instance.

        Returns:
            discord-webhook: The discord-webhook instance.
        """
        return DiscordWebhook(url=self._urls, username=self._username, **kwargs)

    def send_message(self, content: str, rate_limit_retry: bool = False,
                     allowed_mentions: List[str] = None, files: Union[List[str], Dict[str, bytes]] = None,
                     **kwargs) -> Response:
        """
        Sends a simple message.

        Args:
            content (str): The message content.
            rate_limit_retry (bool, optional): if rate_limit_retry is True then in the event that you are
                being rate limited by Discord your webhook will automatically be sent once
                the rate limit has been lifted
            allowed_mentions (list, optional): The list of users to ping.
            files (str): Add files.

        Returns:
            response: Webhook response.
        """
        files_dict = {}
        if isinstance(files, dict):
            files_dict = files
        else:
            for filepath in files or []:
                with open(file=filepath, mode='rb') as f:
                    files_dict[os.path.basename(filepath)] = f.read()

        webhook = DiscordWebhook(url=self._urls, username=self._username, files=files_dict, **kwargs)
        webhook.content = content
        webhook.rate_limit_retry = rate_limit_retry
        if allowed_mentions is not None:
            webhook.allowed_mentions = {"users": allowed_mentions}
        return webhook.execute()

    def send_embedded_message(self, message: EmbeddedMessage, **kwargs) -> Union[Response, List[Response]]:
        """
        Discord Embed Message.

        Args:
            message (EmbeddedMessage): The message content.
                See [EmbeddedMessage][botcity.plugins.discord.models.EmbeddedMessage]

        Returns:
            response: Webhook response.
        """
        if isinstance(message.color, Color):
            color = message.color.value
        else:
            color = message.color
        embed = DiscordEmbed(title=message.title, description=message.description, color=color)

        if message.author is not None:
            embed.set_author(name=message.author.name, url=message.author.url, icon_url=message.author.icon_url)

        if message.image is not None:
            embed.set_image(url=message.image)

        if message.thumbnail is not None:
            embed.set_thumbnail(url=message.thumbnail)

        if message.footer is not None:
            embed.set_footer(text=message.footer.text, icon_url=message.footer.icon_url)

        if message.timestamp is not None:
            embed.set_timestamp(timestamp=message.timestamp)
        else:
            embed.set_timestamp()

        if message.fields is not None:
            for field in message.fields:
                embed.add_embed_field(name=field.name, value=field.value)

        files_dict = {}
        if isinstance(message.files, dict):
            files_dict = message.files
        else:
            for filepath in message.files or []:
                with open(file=filepath, mode='rb') as f:
                    files_dict[os.path.basename(filepath)] = f.read()

        webhook = DiscordWebhook(url=self._urls, username=self._username, files=files_dict, **kwargs)
        webhook.add_embed(embed)
        return webhook.execute()

    def edit_message(self, message_response: Union[Response, List[Response]],
                     new_content_message: str, **kwargs) -> Union[Response, List[Response]]:
        """
        Edits the message based on the response passed as argument.

        Args:
            message_response (requests.Response or list): webhook.execute() response
            new_content_message: The new message content.

        Returns:
            response: Webhook response.
        """
        webhook = DiscordWebhook(url=self._urls, username=self._username, **kwargs)
        webhook.content = new_content_message
        return webhook.edit(message_response, **kwargs)

    def delete_message(self, message_response: Union[Response, List[Response]],
                       **kwargs) -> Union[Response, List[Response]]:
        """
        Deletes the message based on the response passed as argument.

        Args:
            message_response (requests.Response or list): webhook.execute() response

        Returns:
            response: Webhook response.
        """
        webhook = DiscordWebhook(url=self._urls, username=self._username, **kwargs)
        return webhook.delete(message_response)

    def send_file(self, files: List[str], **kwargs):
        """
        Upload file to the webhook.

        Args:
            files (list): The file paths.

        Returns:
            response: Webhook response.
        """
        webhook = DiscordWebhook(url=self._urls, username=self._username, **kwargs)
        for file in files:
            with open(file, "rb") as f:
                webhook.add_file(file=f.read(), filename=os.path.basename(file))
        return webhook.execute()



from setuptools import setup, find_namespace_packages
from os import path
from codecs import open
import versioneer

cur_dir = path.abspath(path.dirname(__file__))

with open(path.join(cur_dir, 'requirements.txt'), 'r') as f:
    requirements = f.read().split()

setup(
    name='botcity-discord-plugin',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_namespace_packages(include=['botcity.plugins.*']),
    url='https://bitbucket.org/botcitydev/bot-discord-plugin-python.git',
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    install_requires=requirements,
    include_package_data=True,
    python_requires='>=3.7'
)



import enum
from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List, Union


@dataclass
class Author:
    """
    Author info.

    Attributes:
        url: Profile url.
        name: Author name.
        icon_url: URL to author profile picture.
    """
    url: str
    name: str
    icon_url: str


@dataclass
class Footer:
    """
    Footer info.

    Attributes:
        text: Footer text.
        icon_url: URL to footer icon.
    """
    text: str
    icon_url: str


@dataclass
class Field:
    """
    Field info.

    Attributes:
        name: Name of field.
        value: Value of field.
        inline: True if the embed fields are placed side by side, False the field are placed in new line.
    """
    name: str
    value: str
    inline: bool = False


class Color(enum.Enum):
    """
    The color type.
    Attributes:
        RED (str): The red color in hexadecimal.
        GRAY (str): The gray color in hexadecimal.
        BLUE (str): The blue color in hexadecimal.
        GREEN (str): The green color in hexadecimal.
        WHITE (str): The white color in hexadecimal.
        BLACK (str): The black color in hexadecimal.
        BROWN (str): The brown color in hexadecimal.
        YELLOW (str): The yellow color in hexadecimal.
        PURPLE (str): The purple color in hexadecimal.
        ORANGE (str): The orange color in hexadecimal.
    """
    RED = 'FF0000'
    GRAY = '808080'
    BLUE = '0000FF'
    GREEN = '008000'
    WHITE = 'FFFFFF'
    BLACK = '000000'
    BROWN = '964B00'
    YELLOW = 'FFFF00'
    PURPLE = '800080'
    ORANGE = 'FFA500'


@dataclass
class EmbeddedMessage:
    """
    The embedded message.

    Attributes:
        title: Title of embed.
        description: Description body of embed.
        color: Color code of the embed as hexadecimal string or
            [Color][botcity.plugins.discord.models.Color] enum.
        image: Your image url here.
        author: [Author][botcity.plugins.discord.models.Author] information.
        footer: [Footer][botcity.plugins.discord.models.Footer] texts.
        thumbnail: Your thumbnail url here.
        fields: List of [Field][botcity.plugins.discord.models.Field] information.
        timestamp: Timestamp of embed content.
        files: Send files in embedded message.
    """
    title: str
    description: str
    color: Union[str, Color]
    image: str = None
    author: Author = None
    footer: Footer = None
    thumbnail: str = None
    fields: List[Field] = None
    timestamp: float = datetime.now().timestamp()
    files: Union[List[str], Dict[str, bytes]] = None
